import os
from uuid import uuid4
import logging
import tempfile
LOGO = r"""
   ,-------------------------------------------------------------------------------.
   |                                                                               |
   |                                                                               |
   |       ____  _       _              _                   ____  _____  ___       |
   |      |  _ \| |_   _| |_ ___  _ __ (_)_   _ _ __ ___   |___ \|___ / ( _ )      |
   |      | |_) | | | | | __/ _ \| '_ \| | | | | '_ ` _ \    __) | |_ \ / _ \      |
   |      |  __/| | |_| | || (_) | | | | | |_| | | | | | |  / __/ ___) | (_) |     |
   |      |_|   |_|\__,_|\__\___/|_| |_|_|\__,_|_| |_| |_| |_____|____/ \___/      |
   |                                                                               |
   |                                                                               |
   |                                                             version 0.2.2     |
   |                                                                               |
   |                 This is a SCA Agent, Copyright@Plutonium Team                 |
   |                                                                               |
   |                                                                               |
   `-------------------------------------------------------------------------------'
"""
# 结果轮询最大时间
RESULT_CHECK_INTERVAL = 1
RESULT_CHECK_TIME_MAX = 1 * 10
# 结果轮询最大次数
RESULT_CHECK_COUNT = 20

# DATA_DIR = os.path.dirname(os.path.abspath(__file__)) +'/data/'
VOYAGER_SERVER = os.getenv('VOYAGER_SERVER') if os.getenv('VOYAGER_SERVER') else 'http://localhost:6600/'
VOYAGER_USERNAME = os.getenv('VOYAGER_USERNAME') if os.getenv('VOYAGER_USERNAME') else 'dev'
VOYAGER_PASSWORD = os.getenv('VOYAGER_PASSWORD') if os.getenv('VOYAGER_PASSWORD') else 'dev'
VOYAGER_TOKEN = os.getenv('VOYAGER_TOKEN') if os.getenv('VOYAGER_TOKEN') else 'x'
AGENT_UUID = uuid4().hex
# 默认忽略的目录
TEMP_DIR = tempfile.gettempdir()
if not TEMP_DIR:
    TEMP_DIR = './tmp/'
    try:
        if not os.path.exists(TEMP_DIR):
            os.makedirs(TEMP_DIR)
    except Exception as e:
        print(e)
DATA_DIR = TEMP_DIR + '/plutonium-{}/'.format(AGENT_UUID)
if not os.path.exists(DATA_DIR):
    try:
        os.makedirs(DATA_DIR)
    except Exception as e:
        DATA_DIR = './tmp'

# 日志配置
if os.getenv("DEBUG"):
    LOG_LEVEL = logging.DEBUG
else:
    LOG_LEVEL = logging.INFO
LOG_FILENAME = DATA_DIR + 'plutonium_run_{}.log'.format(uuid4().hex)
LOG_FORMAT = '%(asctime)s - %(levelname)s - %(filename)s - %(funcName)s  - %(lineno)d - %(message)s'
SCA_TOOLS = {
    'cdxgen': {
        'description': 'cdxgen tool',
        'default_options':[],
    },
    'jd_sca': {
        'descripion': '',
        'result': ''
    },
    'opensca': {
        'descripion': '',
        'result': ''
    },
}

VOYAGER_GET_TOKEN_API="/sbom/api/base/get-auth-token/"
VOYAGER_CHECK_TOKEN_API="/sbom/api/base/check-token/"
VOYAGER_BASE_API="/sbom/api/sbom/gather/"

# 调用命令超时时间，默认5分钟
CMD_TIME_OUT = 5 * 60


import json
import os
import argparse
from plutonium.utils import logger, VoyagerDetect, zip_folder
from plutonium.config import (
    VOYAGER_SERVER,
    VOYAGER_USERNAME,
    VOYAGER_PASSWORD,
    VOYAGER_TOKEN,
    LOG_FILENAME,
    LOGO,
    DATA_DIR,
    AGENT_UUID
)

def init_parse():
    parser = argparse.ArgumentParser(
        description="SCA Agent based on Cdxgen and internal project Voyager I, for application dependencies and risk discovery。"
    )
    parser.add_argument(
        "-t",
        "--type",
        dest="type",
        required=True,
        help="project_sca/image_sca/package_sca/api/other/project_vul/image_vul",
    )
    parser.add_argument(
        "-trigger-source",
        "--trigger-source",
        dest="trigger_source",
        help="Trigger Source",
    )
    parser.add_argument(
        "-s",
        "--source",
        dest="source",
        required=True,
        help="Source directory or container image or binary file",
    )
    parser.add_argument(
        "--tool",
        dest="tool",
        help="Analysis Tool",
    )
    parser.add_argument(
        "--tool-opitons",
        dest="tool_options",
        help="Analysis Tool options",
    )
    parser.add_argument(
        "--firewall-status",
        dest="firewall_status",
        action="store_true",
        default=False,
        help="是否进行拦截检测",
    )
    parser.add_argument(
        "--is-sync",
        dest="is_sync",
        action="store_true",
        default=False,
        help="是否同步检测",
    )
    # 服务端参数
    parser.add_argument(
        "--voyager-server",
        default=VOYAGER_SERVER,
        dest="voyager_server",
        help="Voyager server url. Eg: https://api.voyager.com",
    )
    parser.add_argument(
        "--voyager-username",
        default=VOYAGER_USERNAME,
        dest="voyager_username",
        help="Voyager username",
    )
    parser.add_argument(
        "--voyager-password",
        default=VOYAGER_PASSWORD,
        dest="voyager_password",
        help="Voyager password",
    )
    parser.add_argument(
        "--voyager-token",
        default=VOYAGER_TOKEN,
        dest="voyager_token",
        help="Voyager token for token based submission",
    )
    parser.add_argument(
        "--voyager-api",
        default="",
        dest="voyager_api",
        help="Voyager api url",
    )
    parser.add_argument(
        "--analyze-uuid",
        dest="analyze_uuid",
        help="分析任务id",
    )
    # 项目参数信息
    parser.add_argument(
        "--project-name",
        dest="project_name",
        help="project name",
    )
    parser.add_argument(
        "--project-url",
        dest="project_url",
        help="project repository url",
    )
    parser.add_argument(
        "--project-user",
        dest="project_user",
        help="project user",
    )
    parser.add_argument(
        "--project-branch",
        dest="project_branch",
        help="project branch",
    )
    parser.add_argument(
        "--project-file",
        dest="project_file",
        help="project file",
    )
    parser.add_argument(
        "--project-commit-id",
        dest="project_commit_id",
        help="project commit id",
    )
    # 镜像参数信息
    parser.add_argument(
        "--image-name",
        dest="image_name",
        help="镜像名称",
    )
    parser.add_argument(
        "--image-file",
        dest="image_file",
        help="镜像文件",
    )
    parser.add_argument(
        "--image-repository-url",
        dest="image_repository_url",
        help="镜像仓库地址",
    )
    parser.add_argument(
        "--extra-data",
        dest="extra_data",
        help="附加参数a=b&c=d",
    )
    parser.print_help()
    return parser.parse_args()


def main():
    print(LOGO)
    args = init_parse()
    data = {
        'type': args.type,
        'agent_uuid': AGENT_UUID,
        'trigger_source': args.trigger_source,
        'tool': args.tool,
        'tool_options': args.tool_options,
        'firewall_status': args.firewall_status,
        'is_sync': args.is_sync,
        'analyze_uuid': args.analyze_uuid if args.analyze_uuid else '',
        # 项目信息
        # 镜像类
        'image_name': args.image_name,
        'image_repository_url': args.image_repository_url,
        # 项目类
        'project_name': args.project_name,
        'project_branch': args.project_branch,
        'project_user': args.project_user,
        'project_commit_id': args.project_commit_id,
        'project_url': args.project_url,
        # 附加参数
        'extra_data': args.extra_data,
    }
    attach_files = [
        # 提交的名称前缀需要与scan_log_detail的type一致
        # 项目文件
        # ('log_file', (SCA, open('./todo.md', 'rb'),)),
        # ('core_files_list', ('pom.xml', open('./todo.md', 'rb'), )),
        # ('core_files_list', ('package.json.lock', open('./todo.md', 'rb'), )),
        # ('sbom_files_list', ('sca_cdxgen.json', open('./voyager.json', 'rb'),)),
        # ('sbom_files_list', ('sca_dependency_tree.txt', open('./todo.md', 'rb'), )),
        # ('vul_files_list', ('vul_veinmind.json', open('./todo.md', 'rb'), )),
    ]
    detector = VoyagerDetect(
        token=args.voyager_token if args.voyager_token else VOYAGER_TOKEN,
        url=args.voyager_server if args.voyager_server else VOYAGER_SERVER,
        username=args.voyager_username if args.voyager_username else VOYAGER_USERNAME,
        password=args.voyager_password if args.voyager_password else VOYAGER_PASSWORD,
        api=args.voyager_api if args.voyager_api else ''
    )
    # 项目安全检测
    if args.type in ['project_sca', 'image_sca']:
        # 逐步进行检测
        for tool in args.tool.split(','):
            detector.sca_analysis(args.source, tool)
        # 打包上传
        zip_folder(DATA_DIR)
        target_file = DATA_DIR.rstrip('/')+'.zip'
        try:
            attach_files.append(
                ('attach_file', (target_file.split('/')[-1], open(target_file, 'rb'),)),
            )
        except Exception as e:
            logger.error(e)
        scan_status = detector.upload(data, attach_files)
        print(scan_status)
    # 镜像安全检测
    elif args.type in ['image_vul',]:
        if args.image_file:
            try:
                attach_files.append(
                    ('attach_file', (args.image_file.split('/')[-1].split('.')[0], open(args.image_file, 'rb'),)),
                )
            except Exception as e:
                logger.error(e)
        scan_status = detector.scan(data, attach_files)
        print(scan_status)
    
    else:
        pass
if __name__ == '__main__':
    main()



import subprocess
import logging
import os
import json
import sys
import zipfile
import shutil
import requests
import plutonium.config as config
from uuid import uuid4
logger = logging.getLogger(__name__)
formatter = logging.Formatter(config.LOG_FORMAT)
file_handler = logging.FileHandler(config.LOG_FILENAME)
file_handler.setFormatter(formatter)
console_handler = logging.StreamHandler()
console_handler.setFormatter(formatter)
logger.addHandler(file_handler)
logger.addHandler(console_handler)
logger.setLevel(config.LOG_LEVEL)
# 获取路径
def resource_path(relative_path):
    try:
        base_path = sys._MEIPASS
    except Exception:
        base_path = os.path.dirname(__file__)
    return os.path.join(base_path, relative_path)
# 执行命令
def exec_tool(args, log_file, timeout=config.CMD_TIME_OUT, cwd=None, ):
    result = {
        'status': False,
        'message': '',
        'data': ''
    }
    try:
        logger.debug('Executing "{}"'.format(" ".join(args)))

        if os.environ.get("FETCH_LICENSE"):
            logger.debug(
                "License information would be fetched from the registry. This would take several minutes ..."
            )
        with open(log_file, 'a') as f:
            cp = subprocess.run(
                args,
                stdout=f,
                stderr=subprocess.STDOUT,
                cwd=cwd,
                env=os.environ.copy(),
                check=False,
                shell=False,
                encoding="utf-8",
                timeout=timeout
            )
            logger.debug(cp.stdout)
            result['status'] = True
    except subprocess.TimeoutExpired as timeout_error:
        logger.error(timeout)
        result['message'] = '执行超时-{}'.format(timeout_error)
    except Exception as e:
        logger.exception(e)
        result['message'] = str(e)
        return str(e)
    print(result)
    return result

def zip_folder(file_dir):
    target_file = file_dir+'/../'+file_dir.strip('/').split('/')[-1]+'.zip'
    print(target_file)
    with zipfile.ZipFile(target_file, 'w', zipfile.ZIP_DEFLATED) as zipf:
        for root, dirs, files in os.walk(file_dir):
            for file in files:
                file_path = os.path.join(root, file)
                zipf.write(file_path, os.path.relpath(file_path, file_dir))
    

# 通过cdxgen来生成sbom
def sca_by_cdxgen(bom_file, src_dir=".", timeout=config.CMD_TIME_OUT):
    result = {
        'status': False,
        'data': {
            'cmd': '',
            'result_file': bom_file
        },
        'message': '',
    }
    cdxgen_cmd = os.environ.get("CDXGEN_CMD", "cdxgen")
    if not shutil.which(cdxgen_cmd):
        local_bin = resource_path(
            os.path.join(
                "local_bin", "cdxgen.exe" if sys.platform == "win32" else "cdxgen"
            )
        )
        if not os.path.exists(local_bin):
            result['message'] = 'command not found'
            return result
        try:
            cdxgen_cmd = local_bin
            # Set the plugins directory as an environment variable
            os.environ["CDXGEN_PLUGINS_DIR"] = resource_path("local_bin")
        except Exception as e:
            result['message'] = e
            return result
    sca_args = [cdxgen_cmd, "-o", bom_file]
    sca_args.append(src_dir)
    logger.info(sca_args)
    exec_status = exec_tool(sca_args, config.LOG_FILENAME, timeout )
    result['status'] = exec_status['status']
    result['message'] = exec_status['message']
    result['data']['cmd'] = ' '.join(sca_args)
    return result

def sca_by_opensca(bom_file, src_dir=".", log_file="",  timeout=config.CMD_TIME_OUT):
    # 获取配置文件，配置maven等信息

    result = {
        'status': False,
        'data': {
            'cmd': '',
            'result_file': bom_file
        },
        'message': '',
    }
    sca_args = ["opensca-cli", "-path", src_dir, "-config", config.TEMP_DIR+'/opensca_config.json', "-out", "{}".format(bom_file)]
    exec_status = exec_tool(sca_args, config.LOG_FILENAME, timeout)
    result['status'] = exec_status['status']
    result['message'] = exec_status['message']
    result['data']['cmd'] = ' '.join(sca_args)
    return result
def sca_by_jd_sbom_tool(bom_file, src_dir=".", log_file="",  timeout=config.CMD_TIME_OUT):
    # 获取配置文件，配置maven等信息
    result = {
        'status': False,
        'data': {
            'cmd': '',
            'result_file': bom_file
        },
        'message': '',
    }
    sca_args = ["sbom-tool", "-out ", bom_file, "-path", src_dir, "-log", log_file]
    logger.info(sca_args)
    exec_status = exec_tool(sca_args, config.LOG_FILENAME, timeout)
    result['status'] = exec_status['status']
    result['message'] = exec_status['message']
    result['data']['cmd'] = ' '.join(sca_args)
    return result

def sca_by_fosseye(bom_file, src_dir=".", log_file="",  timeout=config.CMD_TIME_OUT):
    # 获取配置文件，配置maven等信息

    result = {
        'status': False,
        'data': {
            'cmd': '',
            'result_file': bom_file
        },
        'message': '',
    }
    sca_args = ["opensca-cli", "-out ", bom_file, "-path", src_dir, "-log", log_file]
    logger.info(sca_args)
    exec_status = exec_tool(sca_args, config.LOG_FILENAME, timeout)
    result['status'] = exec_status['status']
    result['message'] = exec_status['message']
    result['data']['cmd'] = ' '.join(sca_args)
    return result

# 服务端检测
class VoyagerDetect():
    def __init__(self, token=None, url=None, username=None, password=None, api=None):
        self.api_url = url
        self.api_token = token
        self.api_username = username
        self.api_password = password
        self.req = requests.Session()

    # 1.token生成，token有效期较长，失效后再进行重新生成
    def get_new_token(self):
        try:
            self.req.headers = {}
            res = self.req.post(self.api_url + config.VOYAGER_GET_TOKEN_API,
                                data={'username': self.api_username, 'password': self.api_password})
            token = res.json()['token']
            headers = {
                # 注意Token后有空格
                'Authorization': 'Token ' + token
            }
            self.req.headers = headers
            return token
        except Exception as e:
            logger.error(e)
            return None

    def check_token_valid(self):
        headers = {
            # 注意Token后有空格
            'Authorization': 'Token ' + self.api_token
        }
        self.req.headers = headers
        try:
            res = self.req.get(self.api_url + config.VOYAGER_CHECK_TOKEN_API, )
            if res.status_code == 200 and res.json()['success']:
                return True
        except Exception as e:
            logger.error(e)
        return False

    # 登录认证
    def login(self):
        login_status = {
            'status': False,
            'message': '',
            'data': None
        }
        if self.api_token:
            if self.check_token_valid():
                login_status['status'] = True
                return login_status
            else:
                # token失效，使用账号密码登录
                login_status['message'] = 'token失效，使用账号密码登录'

        if self.api_username and self.api_password:
            # 获取认证token
            token = self.get_new_token()
            # logger.info(token)
            if token:
                self.api_token = token
                headers = {
                    # 注意Token后有空格
                    'Authorization': 'Token ' + self.api_token
                }
                self.req.headers = headers
                login_status['status'] = True
            else:
                login_status['message'] = '无法生成访问token，账号密码可能错误'
        else:
            login_status['message'] = '请提供API账号以及密码信息'
        logger.info(login_status['message'])
        return login_status

    def init_opensca(self):
        login_status = self.login()
        data = {
            'type': 'get_opensca_config',
        }
        if login_status['status']:
            response = self.req.post(self.api_url + config.VOYAGER_BASE_API, data=data,
                                         files=[('files', ('', None,)), ])
            try:
                print(response.json())
                if response.json().get('success'):
                    config_data = response.json().get('data')
                    print(config.TEMP_DIR+'opensca_config.json')
                    with open(config.TEMP_DIR+'/opensca_config.json', 'w') as f:
                        f.write(json.dumps(config_data))
            except Exception as e:
                logger.error(e)
        else:
            return login_status['message']

    # sca分析
    def sca_analysis(self, src_dir=".", tool='cdxgen'):
        if tool == 'cdxgen':
            result_file = config.DATA_DIR+'/{}_{}.json'.format(tool, uuid4().hex)
            return sca_by_cdxgen(result_file, src_dir)
        elif tool == 'opensca':
            self.init_opensca()
            result_file = config.DATA_DIR+'/{}_{}.json'.format(tool, uuid4().hex)
            log_file = config.DATA_DIR+'/{}_log_{}.log'.format(tool, uuid4().hex)
            return sca_by_opensca(result_file, src_dir, log_file)
        elif tool == 'jd-sbom-tool':
            pass
        elif tool == 'fosseye':
            pass
        return None

    # upload
    def upload(self, data, files):
        login_status = self.login()
        if login_status['status']:
            if files:
                response = self.req.post(self.api_url + config.VOYAGER_BASE_API, data=data, files=files)
            else:
                response = self.req.post(self.api_url + config.VOYAGER_BASE_API, data=data,
                                         files=[('files', ('', None,)), ])
            return response.json()
        else:
            return login_status['message']
    # 获取扫描结果或报告
    def get_scan_result(self, task_id):
        login_status = self.login()
        if login_status['status']:
            response = self.req.get(self.api_url + VOYAGER_BASE_API+'?task_id={}'.format(task_id),)
            return response.json()
        else:
            return login_status
    
    # 新版的与服务端交互
    def scan(self, data, files):
        login_status = self.login()
        if login_status['status']:
            if files:
                response = self.req.post(self.api_url + config.VOYAGER_BASE_API, data=data, files=files)
            else:
                response = self.req.post(self.api_url + config.VOYAGER_BASE_API, data=data, files=[('files', ('', None,)), ])
            return response.json()
        else:
            return login_status['message']
    

    